public class TANode {
    private String key, value;
    private TANode left, right;

    public TANode(String k, String v) {
        key = k;
        value = v;
    }

    private int compare(String k) {
        if (k == null) {
            return key == null ? 0 : -1;
        }
        if (key == null) {
            return 1;
        }
        return k.compareTo(key);
    }

    public String put(String k, String v) {
        int cmp = compare(k);
        if (cmp < 0) {
            if (left != null) {
                return left.put(k, v);
            }
            left = new TANode(k, v);
        } else if (cmp > 0) {
            if (right != null) {
                return right.put(k, v);
            }
            right = new TANode(k, v);
        } else {
            String result = value;
            value = v;
            return result;
        }
        return null;
    }

    public TANode find(String k) {
        int cmp = compare(k);
        if (cmp == 0) {
            return this;
        }
        TANode node = cmp < 0 ? left : right;
        if (node == null) {
            return null;
        }
        return node.find(k);
    }

    public boolean hasValue(String v) {
        return (v == null ? value == v
                : v.equals(value)) ||
                (left != null && left.hasValue(v)) ||
                (right != null && right.hasValue(v));
    }

    public String value() {
        return value;
    }


    public String toString() {
        String s = "(" + key + ", " + value + ")";
        if (left != null) {
            s = left + ", " + s;
        }
        if (right != null) {
            s += ", " + right;
        }
        return s;
    }

    public TANode asList(TANode list) {
        if (left != null) {
            list = left.asList(list);
        }
        TANode next = list;
        list = new TANode(key, value);
        list.left = next;
        if (right != null) {
            list = right.asList(list);
        }
        return list;
    }

    public TANode equalsList(TANode list) {
        if (list == null || (right != null && (list = right.equalsList(list)) == null)) {
            return null;
        }
        if ((key == null ? list.key != null : !key.equals(list.key)) ||
                (value == null ? list.value != null : !value.equals(list.value))) {
            return null;
        }
        list = list.left;
        if (left != null) {
            list = left.equalsList(list);
        }
        return list;
    }

    public int hashCodeSum() {
        int i = 582764572;
        if (key != null) {
            i += key.hashCode();
        }
        if (value != null) {
            i += value.hashCode();
        }
        if (left != null) {
            i += left.hashCodeSum();
        }
        if (right != null) {
            i += right.hashCodeSum();
        }
        return i;
    }

}
