import java.util.Arrays;

public class HashTab implements Assoc, Map {
    private String[] ks;
    private String[] vs;
    private int count;

    public HashTab() {
        clear();
    }

    public void clear() {
        ks = new String[65];
        vs = new String[65];
        count = 0;
    }

    private int find(String k) {
        if (k == null) {
            return ks.length - 1;
        }
        int i = k.hashCode() & (ks.length - 2);
        while (ks[i] != null && !ks[i].equals(k)) {
            i = (i + 1) & (ks.length - 2);
        }
        return i;
    }

    public String put(String k, String v) {
        if (k == null || v == null) {
            return null;
        }
        int i = find(k);
        String old = vs[i];
        vs[i] = v;
        if (ks[i] == null) {
            ks[i] = k;
            if (++count >= ks.length * 3 / 4) {
                String[] oks = ks, ovs = vs;
                ks = new String[(oks.length << 1) - 1];
                vs = new String[(oks.length << 1) - 1];
                for (int j = 0; j < oks.length; j++) {
                    if (oks[j] != null) {
                        ks[i = find(oks[j])] = oks[j];
                        vs[i] = ovs[j];
                    }
                }
            }
        }
        return old;
    }

    public String get(String k) {
        return vs[find(k)];
    }

    public boolean containsKey(String k) {
        return ks[find(k)] != null;
    }

    private int indexOfValue(String v, int from) {
        if (v != null) {
            for (int i = from; i < vs.length - 1; i++) {
                if (v.equals(vs[i])) {
                    return i;
                }
            }
        }
        return -1;
    }

    public boolean containsValue(String v) {
        return indexOfValue(v, 0) >= 0;
    }

    private void del(int i) {
        if (i < ks.length - 1) {
            ks[i] = vs[i] = null;
            count--;
            for (i = (i + 1) & (ks.length - 2);
                 ks[i] != null;
                 i = (i + 1) & (ks.length - 2)) {
                String ki = ks[i], vi = vs[i];
                ks[i] = vs[i] = null;
                count--;
                put(ki, vi);
            }
        }

    }

    public String remove(String k) {
        int i = find(k);
        String result = vs[i];
        del(i);
        return result;
    }

    public int size() {
        return count;
    }

    public Collection keys() {
        return new KeySet(this);
    }

    public Collection values() {
        return new ValueSet(this);
    }

    public int keyArray(String[] keys) {
        int i = 0, j = -1;
        while (i < keys.length && ++j < ks.length - 1) {
            if (ks[j] != null) {
                keys[i++] = ks[j];
            }
        }
        return i;
    }

    public int valueArray(String[] values) {
        int i = 0, j = -1;
        while (i < values.length && ++j < vs.length - 1) {
            if (vs[j] != null) {
                values[i++] = vs[j];
            }
        }
        return i;
    }

    public int countValues(String value) {
        int count = 0, i = -1;
        while ((i = indexOfValue(value, i + 1)) >= 0) {
            count++;
        }
        return count;
    }

    public int removeValues(String value) {
        int count = 0, i = 0;
        while ((i = indexOfValue(value, i)) >= 0) {
            del(i);
            count++;
        }
        return count;
    }

    public Iterator keyIterator() {
        return new HashTabIter(ks);
    }

    public Iterator valueIterator() {
        return new HashTabIter(vs);
    }

    public String toString() {
        String s = "";
        for (int i = 0; i < ks.length - 1; i++) {
            if (ks[i] != null) {
                s += (s.isEmpty() ? "(" : ", (") + ks[i] + ", " + vs[i] + ")";
            }
        }
        return "{" + s + "}";
    }

    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || o.getClass() != HashTab.class) {
            return false;
        }
        HashTab t = (HashTab) o;
        if (count != t.count) {
            return false;
        }
        for (int i = 0; i < ks.length - 1; i++) {
            if (ks[i] != null && !vs[i].equals(t.get(ks[i]))) {
                return false;
            }
        }
        return true;
    }

    public int hashCode() {
        int h = count;
        for (int i = 0; i < ks.length - 1; i++) {
            if (ks[i] != null) {
                h += ks[i].hashCode() + count * vs[i].hashCode();
            }
        }
        return h;
    }



    public static void main(String[] args) {
        Map map = new HashTab();
        for (char c = 'a'; c <= 'z'; c++) {
            map.put("k" + c, "v" + c);
        }
        System.out.println("map: " + map);
        System.out.println("map.size(): " + map.size());
        Collection ks = map.keys(), vs = map.values();
        System.out.println("ks: " + ks);
        System.out.println("vs: " + vs);
        map.remove("kh");
        map.remove("ka");
        map.remove("kz");
        System.out.println("map: " + map);
        System.out.println("ks: " + ks);
        System.out.println("vs: " + vs);
        System.out.println("ks.size(): " + ks.size());
        System.out.println("vs.size(): " + vs.size());
        System.out.println("Arrays.toString(ks.toArray()): " + Arrays.toString(ks.toArray()));
        System.out.println("Arrays.toString(vs.toArray()): " + Arrays.toString(vs.toArray()));
        System.out.println("ks.removeAll(\"kk\"): " + ks.removeAll("kk"));
        System.out.println("ks.removeAll(\"\"): " + ks.removeAll(""));
        System.out.println("ks.removeAll(null): " + ks.removeAll(null));
        System.out.println("vs.removeAll(\"vk\"): " + vs.removeAll("vk"));
        System.out.println("vs.removeAll(\"vj\"): " + vs.removeAll("vj"));
        System.out.println("vs.removeAll(null): " + vs.removeAll(null));
        System.out.println("map: " + map);
        System.out.println("ks: " + ks);
        System.out.println("vs: " + vs);
        System.out.println("map.containsKey(\"kk\"): " + map.containsKey("kk"));
        System.out.println("map.containsKey(\"ki\"): " + map.containsKey("ki"));
        System.out.println("ks.contains(\"kk\"): " + ks.contains("kk"));
        System.out.println("ks.contains(\"ki\"): " + ks.contains("ki"));
        System.out.println("map.containsValue(\"vk\"): " + map.containsValue("vk"));
        System.out.println("map.containsValue(\"vi\"): " + map.containsValue("vi"));
        System.out.println("vs.contains(\"vk\"): " + vs.contains("vk"));
        System.out.println("vs.contains(\"vi\"): " + vs.contains("vi"));
        System.out.println("map.put(\"putk\", \"putv\"): " + map.put("putk", "putv"));
        System.out.println("map.put(\"putv\", \"putv\"): " + map.put("putv", "putv"));
        System.out.println("map.put(\"kk\", \"kv\"): " + map.put("kk", "kv"));
        System.out.println("map.put(\"kb\", \"putv\"): " + map.put("kb", "putv"));
        System.out.println("vs.contains(\"putv\"): " + vs.contains("putv"));
        vs.clear();
        System.out.println("map: " + map);
        System.out.println("ks: " + ks);
        System.out.println("vs: " + vs);
    }

}
