import java.io.*;
import java.util.*;
import java.util.Map;
import java.util.regex.Pattern;

public class StudInfo {
    private final static Pattern
      SEP = Pattern.compile(";"),
      PAT = Pattern.compile("\\d{8};\\w(\\w| )*;\\d+");
    private String name;
    private int points;

    private StudInfo(String n) { name = n; }

    public static void main(String[] args) {
        try {
            if (args.length < 2) {
                throw new IOException("Zu wenige Kommandozeilenargumente.\nVerwendung: java StudInfo <Eingabedatei> [<Eingabedatei> ...] <Ausgabedatei>");
            }
            Map<Long, StudInfo> map = new HashMap<>();
            Set<String> used = new HashSet<>();
            for (int i = 0; i < args.length-1; i++) {
                if (used.contains(args[i])) {
                    throw new IOException(args[i] + " kommt mehrfach als Eingabedatei vor");
                }
                used.add(args[i]);
                if (args[i].equals("-")) {
                    add(map, System.in, "Standardeingabe");
                }
                else {
                    try (InputStream s = new FileInputStream(args[i])) {
                        add(map, s, args[i]);
                    }
                }
            }
            if (args[args.length - 1].equals("-")) {
                write(map, System.out);
            }
            else {
                try (PrintStream p = new PrintStream(args[args.length-1])) {
                    write(map, p);
                }
            }
        } catch (IOException e) {
            System.err.println("Fehler: " + e.getMessage());
        }
    }

    private static void add(Map<Long, StudInfo> map, InputStream in, String fileName) throws IOException {
        Set<Long> used = new HashSet<>();
        Scanner lines = new Scanner(in);
        while (lines.hasNextLine()) {
            String line = lines.nextLine();
            if (!line.isEmpty()) {
                if (!PAT.matcher(line).matches()) {
                    throw new IOException("Falsches Zeilenformat in " + fileName + ": " + line + "\nErwartetes Format: <Matrikelnummer>;<Name wie in TISS>;<Punktezahl>");
                }
                Scanner s = new Scanner(line);
                s.useDelimiter(SEP);
                long n = s.nextLong();
                String name = s.next();
                int points = s.nextInt();
                if (used.contains(n)) {
                    throw new IOException("Mehrere Einträge mit gleicher Matrikelnummer " + n + " in " + fileName);
                }
                used.add(n);
                StudInfo i = map.getOrDefault(n, new StudInfo(name));
                if (!i.name.equals(name)) {
                    throw new IOException("Person mit Matrikelnummer " + n + " kommt unter unterschiedlichen Namen vor: " + i.name + " und " + name);
                }
                i.points += points;
                map.put(n, i);
            }
        }
    }

    private static void write(Map<Long, StudInfo> map, PrintStream s) {
        Long[] ns = new Long[map.size()];
        map.keySet().toArray(ns);
        Arrays.sort(ns);
        for (Long n : ns) {
            StudInfo i = map.get(n);
            s.printf("%08d;%s;%d\n",n,i.name,i.points);
        }
    }
}
